
const gulp = require('gulp')
const chalk = require('chalk')
const ora = require('ora')
const qrcode = require('qrcode-terminal')
const webserver = require('gulp-webserver')
const createIoingDemo = require('gulp-create-ioing-demo')
const runSequence = require('run-sequence')
const { PATH, BROWSER, CONFIG_SOURCE, CONFIG_SOURCE_TS } = require('./build/config')

// clean bulid
require('./build/clean-build')

// copy to /static
require('./build/copy-static')

// parse/frameworks
require('./build/parse-frameworks')

// parse/modules
require('./build/parse-modules')

// compile/config
require('./build/compile-config')

// bulid index page
require('./build/compile-index')

// prefetch/modules
require('./build/prefetch-modules')

// service/worker js
require('./build/service-worker')

// compile/script
require('./build/compile-script')

// compile/css
require('./build/compile-style')

// compile/source
require('./build/compile-source')

// compile/source
require('./build/update-version')

// C. creat demo : new --module [name]
gulp
  .task("template", () => {
    createIoingDemo((id, type) => {
      if (type === 'components') {
        console.log(chalk.green(`template write <${id}></${id}> view components`))
      }

      BROWSER.hash = '#' + id
      gulp.start('dev')
    })
  })

// S. startup application
gulp
  .task("webserver", () => {
    // 是否是 ip 访问
    if (parseInt(BROWSER.host) >= 0) {
      qrcode.generate('http' + (BROWSER.https ? 's' : '') + '://' + BROWSER.host + ':' + 8000 + '/' + BROWSER.href + BROWSER.hash, { small: false })
    }

    let browser = BROWSER
    browser.open = browser.open === undefined
                 ? BROWSER.href + BROWSER.hash
                 : browser.open

    return gulp
      .src(PATH.pages)
      .pipe(webserver(browser))
  })

gulp
  .watch(CONFIG_SOURCE.concat(CONFIG_SOURCE_TS), function (event) {
    console.log(chalk.yellow('File ' + event.path + ' was ' + event.type + ', running tasks...'))
    runSequence("compile/config.ts", "compile/config")
  })

gulp
  .task("build", () => {
    process.env.NODE_ENV = 'production'
    require("./build/update-version")()
    gulp.start("dev")
  })

gulp.task('compile', ["clean/build", "copy/static", "compile/style", "compile/source", "compile/index"])

gulp
  .task('compile', function (callback) {
    runSequence("clean/build",
      ["copy/static", "compile/style", "compile/source"],
      "compile/script",
      "compile/script.ts",
      "compile/config",
      "compile/config.ts",
      "parse/frameworks",
      "parse/modules",
      ["prefetch/modules", "prefetch/modules/source", "prefetch/modules/style"],
      "service-worker",
      "compile/index",
      callback)
  })

gulp
  .task("dev", function (callback) {
    runSequence("compile", "webserver")
  })

gulp.task("default", ["dev"])

module.exports = { gulp }
