const gulp = require('gulp')
const chalk = require('chalk')
const ora = require('ora')
const ts = require('gulp-typescript')
const babel = require('gulp-babel')
const uglify = require('gulp-uglify')
const through = require('through2')
const toCJSTransformer = require('gulp-imports-to-commonjs')
const { PATH, CONFIG_SOURCE, CONFIG_SOURCE_TS } = require('./config')

gulp
  .task("compile/config", () => {
    const spinner = ora(chalk.white.bgGreen.bold('compile/config'))
    spinner.start()

    stream = gulp
      .src(CONFIG_SOURCE)
      .on('end', () => {
        spinner.stop()
      })
      .pipe(toCJSTransformer())
      .pipe(babel())

    if (process.env.NODE_ENV === 'production') {
      stream = stream.pipe(uglify())
    }

    return stream.pipe(gulp.dest(PATH.static))
  })

gulp
  .task("compile/config.ts", () => {
    const spinner = ora(chalk.white.bgGreen.bold('compile/config'))
    spinner.start()

    stream = gulp
      .src(CONFIG_SOURCE_TS)
      .on('end', () => {
        spinner.stop()
      })
      .pipe(ts())
      .pipe(toCJSTransformer())
      .pipe(through.obj(function (file, enc, cb, contents) {
        contents = `
(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(["exports"], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports);
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports);
    global.config = mod.exports;
  }
})(this, function (exports) {
${file.contents.toString()}
})
      `
        file.contents = new Buffer(contents)
        this.push(file)

        cb()
      }))

    if (process.env.NODE_ENV === 'production') {
      stream = stream.pipe(uglify({
        mangle: true,
        compress: true
      }))
    }

    return stream.pipe(gulp.dest(PATH.static))
  })
