const fs = require('fs')
const ip = require('ip')
const gitr = require('git-rev-sync')
const objectAssign = require('object-assign')

let config = require('../../build.config')
let webpackConfig = require('../../webpack.config')

if (fs.existsSync(`${process.env.PWD}/build.config.js`)) {
  config = require(`${process.env.PWD}/build.config`)
}

if (fs.existsSync(`${process.env.PWD}/webpack.config.js`)) {
  webpackConfig = require(`${process.env.PWD}/webpack.config`)
}

const version = (() => {
  let v = null
  try {
    v = gitr.branch()
    // branch like: publish/1.1.0
    v = v.indexOf('publish/') > -1 ? v.replace(/publish\//, '') : v
  } catch (e) { }
  return v
})()

const PATH = {
  root: config.root || "static/", // ioing.js 相对 index.html 的引用路径
  pwd: process.env.PWD,
  build: process.env.PWD + '/' + (config.build || "build/"), // build 目录
  pages: process.env.PWD + '/' + (config.pages || "build/"), // 应用 index.html 位置
  static: process.env.PWD + '/' + (config.static || "build/static/") // 应用 cdn 资源位置
}

const BROWSER = objectAssign(
  {
    host: ip.address(),
    href: "index.html",
    hash: "",
    port: 8000,
    https: false,
    directoryListing: true
  },
  config.browser
)

const webpack_config = objectAssign(
  {
    module: {
      rules: [{
        test: /\.js$/,
        exclude: /(node_modules)/,
        use: [{
          loader: 'babel-loader',
          //如果有这个设置则不用在添加.babelrc
          options: {
            presets: [['env', { modules: false }]],
            plugins: ['syntax-dynamic-import']
          }
        }]
      }]
    },
    performance: {
      hints: false
    }
  },
  webpackConfig
)

const PREFETCH_ALL = config.bundle === 'all' ? true : false
const PREFETCH_ID = config.bundle.length ? ["frameworks"].concat(config.bundle) : ["frameworks"]

const CONFIG_SOURCE = [`{components,modules}/**/config.js`]
const CONFIG_SOURCE_TS = [`{components,modules}/**/config.ts`]
const CSS_SOURCE = [`{components,modules}/**/*.css`]
const JS_SOURCE = [`{components,modules}/**/*.js", "!{components,modules}/**/{config, webpack.config}.js`]
const TS_SOURCE = [`{components,modules}/**/*.ts", "!{components,modules}/**/{config, webpack.config}.ts`]
const TEMPLATE_SOURCE = [`{components,modules}/**/*.html`]

let PREFETCH_MODULES = []
let PREFETCH_MODULES_STYLE = []
let PREFETCH_MODULES_SOURCE = []

const PREFETCH_MODULES_STYLE_MAP = {}
const PREFETCH_MODULES_SOURCE_MAP = {}

// 写入路径配置
const LOADCACHE = [
  `
<!-- version: ${version} -->
<script id="ioing" src="ioing.js" root="${PATH.root}"></script>
  `
]


module.exports = {
  PATH,
  BROWSER,
  PREFETCH_ALL,
  PREFETCH_ID,
  CONFIG_SOURCE,
  CONFIG_SOURCE_TS,
  PREFETCH_MODULES,
  PREFETCH_MODULES_STYLE,
  PREFETCH_MODULES_SOURCE,
  PREFETCH_MODULES_STYLE_MAP,
  PREFETCH_MODULES_SOURCE_MAP,
  TEMPLATE_SOURCE,
  CSS_SOURCE,
  JS_SOURCE,
  TS_SOURCE,
  LOADCACHE,
  version,
  config,
  webpack_config
}
