const gulp = require('gulp')
const fs = require('fs')
const chalk = require('chalk')
const ora = require('ora')
const through = require('through2')
const objectAssign = require('object-assign')
const buildWebpack = require('./utils/webpack')
const parseURI = require('./utils/parse-uri')
const { PATH, PREFETCH_ALL, PREFETCH_ID, CONFIG_SOURCE, PREFETCH_MODULES, PREFETCH_MODULES_SOURCE, PREFETCH_MODULES_SOURCE_MAP, PREFETCH_MODULES_STYLE, PREFETCH_MODULES_STYLE_MAP } = require('./config')

gulp
  .task("parse/modules", (cb) => {
    const spinner = ora(chalk.white.bgGreen.bold('parse/modules'))
    spinner.start()

    gulp
      .src(PATH.static + CONFIG_SOURCE[0])
      .on('end', () => {
        spinner.stop()

        cb()
      })
      .pipe(through.obj(function (file, enc, cb, contents) {
        const module_id = file.path.split('/').splice(-2, 1)[0]
        const module_dsl = require(file.path).default
        const module_res = module_dsl.resources
        const module_conf = module_dsl.config
        const config_source = module_conf.source
        const config_style = module_conf.style
        const config_script = module_conf.script
        const style_map = {}
        const source_map = {}
        const script_map = {}
        const style_list = []
        const source_list = []
        const script_list = []

        let tack = 0

        if (module_res) {
          if (config_script.length) {
            for (let name of config_script) {
              let script_src = parseURI(module_res.script[name])
              let script_res = script_src[0]

              if (module_res.script && script_res) {
                script_map[script_res] = name
                script_list.push(`modules/${module_id}/${script_res}`)

                tack++
                buildWebpack(script_map.name, () => {
                  tack--
                  console.log(chalk.green(`build: modules/${module_id}/${script_res}`))
                  if (tack_index === 0) cb()
                })
              } else {
                console.log(`Cannot read property (config.js:resource['script']) of undefined`)
              }
            }
          }

          if (config_source.length) {
            for (let name of config_source) {
              let source_src = parseURI(module_res.source[name])
              let source_res = source_src[0]

              if (module_res.source && source_res) {
                source_map[source_res] = name
                source_list.push(`modules/${module_id}/${source_res}`)
              } else {
                console.log(`Cannot read property (config.js:resource['source']) of undefined`)
              }
            }
          }

          if (config_style.length) {
            for (let name of config_style) {
              let style_src = parseURI(module_res.style[name])
              let style_res = style_src[0]

              if (module_res.style && style_res) {
                style_map[style_res] = name
                style_list.push(`modules/${module_id}/${style_res}`)
              } else {
                console.log(`Cannot read property (config.js:resource['style']) of undefined`)
              }
            }
          }

          if (PREFETCH_ALL || PREFETCH_ID.indexOf(module_id) !== -1) {
            PREFETCH_MODULES.push({
              id: module_id,
              style: style_list,
              source: source_list,
              script: script_list,
              style_map: style_map,
              source_map: source_map,
              script_map: script_map
            })

            for (let i of source_list) {
              PREFETCH_MODULES_SOURCE.push(i)
            }
            PREFETCH_MODULES_SOURCE_MAP[module_id] = source_map

            for (let i of style_list) {
              PREFETCH_MODULES_STYLE.push(i)
            }
            PREFETCH_MODULES_STYLE_MAP[module_id] = style_map
          }
        }

        if (tack === 0) cb()
      }))
  })