const gulp = require('gulp')
const fs = require('fs')
const chalk = require('chalk')
const through = require('through2')
const { PATH, LOADCACHE, PREFETCH_MODULES, PREFETCH_MODULES_STYLE, PREFETCH_MODULES_STYLE_MAP, PREFETCH_MODULES_SOURCE, PREFETCH_MODULES_SOURCE_MAP } = require('./config')

// P. prefetch index modules
gulp
  .task("prefetch/modules", (cb) => {
    if (process.env.NODE_ENV !== 'production') return cb()

    let module_config = []
    for (let module of PREFETCH_MODULES) {
      module_config.push(`${PATH.static}modules/${module.id}/config.js`)
    }

    gulp
      .src(module_config)
      .on('end', cb)
      .pipe(through.obj(function (file, enc, cb, contents) {
        const module_id = file.path.split('/').splice(-2, 1)[0]

        contents = file.contents.toString()
        contents = contents.replace(/define\(/, function (d) {
          return `${d}App.config.root + "modules/${module_id}/config",`
        })

        console.log(chalk.green(`√ compiled: modules/${module_id}/config`))

        LOADCACHE.push('<script>\n' +
          contents + '\n' +
          '</script>')

        cb()
      }))
  })

// P. prefetch/modules/source
gulp
  .task("prefetch/modules/source", (cb) => {
    if (process.env.NODE_ENV !== 'production') return cb()

    let module_source = []
    for (let path of PREFETCH_MODULES_SOURCE) {
      module_source.push(`${PATH.static}${path}`)
    }

    gulp
      .src(module_source)
      .on('end', cb)
      .pipe(through.obj(function (file, enc, cb, contents) {
        const module_id = file.path.split('/').splice(-2, 1)[0]
        const source_path = file.path.split(`${PATH.static}modules/${module_id}/`)[1]

        contents = file.contents.toString()
        contents.replace(/<(([a-z]*\-[a-z]*)(.*?)(?=>))>/ig, function (tag, str, name, attr) {
          let source_content
          let uri = /path\=([\"\'])(.*?)(?=\1)/i.exec(attr)
          uri = uri ? `${PATH.static}/modules/${module_id}/${uri[2]}/${name}/index.html`
            : `${PATH.static}/components/${name}/index.html`


          // 如果文件存在
          if (fs.existsSync(uri)) {
            // 读取文件内容
            source_content = fs.readFileSync(uri, 'utf-8')

            // 如果资源设置了指定路径
            if (uri) {
              source_content = `
<template id="${module_id}/source/${name}" style="display:none">
${source_content}
</template>
              `
              console.log(chalk.green(`√ compiled: modules/${module_id}/source/${name}`))
              // 默认组件资源地址
            } else {
              source_content = `
<template id="components/source/${name}" style="display:none">
${source_content}
</template>
              `
              console.log(chalk.green(`√ compiled: components/source/${name}`))
            }



            LOADCACHE.push('\n' + source_content + '\n')
          } else {
            console.log(chalk.red(`file: ${uri} does not exit and cannot be packaged!`))
          }

          return tag
        })

        contents = `
<template id="${module_id}/source/${PREFETCH_MODULES_SOURCE_MAP[module_id][source_path]}" style="display:none">
${contents}
</template>
        `

        LOADCACHE.push('\n' + contents + '\n')

        cb()
      }))

  })

gulp
  .task("prefetch/components", (cb) => {
    if (process.env.NODE_ENV !== 'production') return cb()

    gulp
      .src(PREFETCH_MODULES_SOURCE)
      .on('end', cb)
      .pipe(through.obj(function (file, enc, cb, contents) {
        const module_id = file.path.split('/').splice(-2, 1)[0]
        const source_path = file.path.split(`${PATH.pwd}/modules/${module_id}/`)[1]

        contents = `
<template id="${module_id}/source/${PREFETCH_MODULES_SOURCE_MAP[module_id][source_path]}" style="display:none">
${file.contents.toString()}
</template>
        `
        console.log(chalk.green(`√ compiled: ${module_id}/components/source/${name}`))

        LOADCACHE.push('\n' + contents + '\n')

        cb()
      }))
  })

// P. prefetch/modules/style
gulp
  .task("prefetch/modules/style", (cb) => {
    if (process.env.NODE_ENV !== 'production') return cb()

    let stream = gulp
      .src(PREFETCH_MODULES_STYLE)
      .on('end', cb)
      .pipe(through.obj(function (file, enc, cb, contents) {
        const module_id = file.path.split('/').splice(-2, 1)[0]
        const source_path = file.path.split(`${PATH.pwd}/modules/${module_id}/`)[1]
        const name = PREFETCH_MODULES_STYLE_MAP[module_id][source_path]
        contents = `
<template id="${module_id}/style/${name}" style="display:none">
${file.contents.toString()}
</template>
        `

        console.log(chalk.green(`√ compiled: modules/${module_id}/style/${name}`))

        LOADCACHE.push('\n' + contents + '\n')

        cb()
      }))
  })
